@@ -22,7 +22,7 @@ from six.moves import urllib
 
 from keystonemiddleware.auth_token import _auth
 from keystonemiddleware.auth_token import _exceptions as ksm_exceptions
-from keystonemiddleware.i18n import _, _LE, _LI, _LW
+from keystonemiddleware.i18n import _
 
 
 def _convert_fetch_cert_exception(fetch_cert):
@@ -44,7 +44,7 @@ class _RequestStrategy(object):
     def __init__(self, adap, include_service_catalog=None):
         self._include_service_catalog = include_service_catalog
 
-    def verify_token(self, user_token):
+    def verify_token(self, user_token, allow_expired=False):
         pass
 
     @_convert_fetch_cert_exception
@@ -73,7 +73,8 @@ class _V2RequestStrategy(_RequestStrategy):
         super(_V2RequestStrategy, self).__init__(adap, **kwargs)
         self._client = v2_client.Client(session=adap)
 
-    def verify_token(self, token):
+    def verify_token(self, token, allow_expired=False):
+        # NOTE(jamielennox): allow_expired is ignored on V2
         auth_ref = self._client.tokens.validate_access_info(token)
 
         if not auth_ref:
@@ -100,10 +101,11 @@ class _V3RequestStrategy(_RequestStrategy):
         super(_V3RequestStrategy, self).__init__(adap, **kwargs)
         self._client = v3_client.Client(session=adap)
 
-    def verify_token(self, token):
+    def verify_token(self, token, allow_expired=False):
         auth_ref = self._client.tokens.validate(
             token,
-            include_catalog=self._include_service_catalog)
+            include_catalog=self._include_service_catalog,
+            allow_expired=allow_expired)
 
         if not auth_ref:
             msg = _('Failed to fetch token data from identity server')
@@ -145,16 +147,18 @@ class IdentityServer(object):
         self._request_strategy_obj = None
 
     @property
-    def auth_uri(self):
-        auth_uri = self._adapter.get_endpoint(interface=plugin.AUTH_INTERFACE)
+    def www_authenticate_uri(self):
+        www_authenticate_uri = self._adapter.get_endpoint(
+            interface=plugin.AUTH_INTERFACE)
 
         # NOTE(jamielennox): This weird stripping of the prefix hack is
         # only relevant to the legacy case. We urljoin '/' to get just the
         # base URI as this is the original behaviour.
         if isinstance(self._adapter.auth, _auth.AuthTokenPlugin):
-            auth_uri = urllib.parse.urljoin(auth_uri, '/').rstrip('/')
+            www_authenticate_uri = urllib.parse.urljoin(
+                www_authenticate_uri, '/').rstrip('/')
 
-        return auth_uri
+        return www_authenticate_uri
 
     @property
     def auth_version(self):
@@ -191,19 +195,20 @@ class IdentityServer(object):
                 return klass
 
         versions = ['v%d.%d' % s.AUTH_VERSION for s in _REQUEST_STRATEGIES]
-        self._LOG.error(_LE('No attempted versions [%s] supported by server'),
+        self._LOG.error('No attempted versions [%s] supported by server',
                         ', '.join(versions))
 
         msg = _('No compatible apis supported by server')
         raise ksm_exceptions.ServiceError(msg)
 
-    def verify_token(self, user_token, retry=True):
+    def verify_token(self, user_token, retry=True, allow_expired=False):
         """Authenticate user token with identity server.
 
         :param user_token: user's token id
         :param retry: flag that forces the middleware to retry
                       user authentication when an indeterminate
                       response is received. Optional.
+        :param allow_expired: Allow retrieving an expired token.
         :returns: access info received from identity server on success
         :rtype: :py:class:`keystoneauth1.access.AccessInfo`
         :raises exc.InvalidToken: if token is rejected
@@ -211,25 +216,27 @@ class IdentityServer(object):
 
         """
         try:
-            auth_ref = self._request_strategy.verify_token(user_token)
+            auth_ref = self._request_strategy.verify_token(
+                user_token,
+                allow_expired=allow_expired)
         except ksa_exceptions.NotFound as e:
-            self._LOG.warning(_LW('Authorization failed for token'))
-            self._LOG.warning(_LW('Identity response: %s'), e.response.text)
+            self._LOG.warning('Authorization failed for token')
+            self._LOG.warning('Identity response: %s', e.response.text)
             raise ksm_exceptions.InvalidToken(_('Token authorization failed'))
         except ksa_exceptions.Unauthorized as e:
-            self._LOG.info(_LI('Identity server rejected authorization'))
-            self._LOG.warning(_LW('Identity response: %s'), e.response.text)
+            self._LOG.info('Identity server rejected authorization')
+            self._LOG.warning('Identity response: %s', e.response.text)
             if retry:
-                self._LOG.info(_LI('Retrying validation'))
+                self._LOG.info('Retrying validation')
                 return self.verify_token(user_token, False)
             msg = _('Identity server rejected authorization necessary to '
                     'fetch token data')
             raise ksm_exceptions.ServiceError(msg)
         except ksa_exceptions.HttpError as e:
             self._LOG.error(
-                _LE('Bad response code while validating token: %s'),
+                'Bad response code while validating token: %s',
                 e.http_status)
-            self._LOG.warning(_LW('Identity response: %s'), e.response.text)
+            self._LOG.warning('Identity response: %s', e.response.text)
             msg = _('Failed to fetch token data from identity server')
             raise ksm_exceptions.ServiceError(msg)
         else:
